"use server";

import { notFound } from "next/navigation";
import { Post, Summary } from "./types";
import { PrismaClient } from "@prisma/client";

export async function getTags(): Promise<string[]> {
  const prisma = new PrismaClient();
  const tags = (await prisma.tag.findMany({ select: { name: true } })).map(
    (tag) => tag.name
  );
  return tags;
}

const PAGE_SIZE = 10;

export async function getPost(slug: string): Promise<Post | null> {
  const primsa = new PrismaClient();
  const result = await primsa.post.findUnique({
    where: { slug: slug },
    include: { tags: { select: { name: true } } },
    omit: {
      contentMarkdown: true,
    },
  });
  if (result == null) {
    notFound();
  }
  const post = {
    ...result,
    tags: result.tags.map((tag) => tag.name),
    contentRendered: result.contentRendered as { __html: string },
    contentMarkdown: undefined,
  };
  return post;
}

export async function getSummaries(
  pageNumber: number,
  tagFilter: string | null = null
): Promise<{ metadata: Summary[]; numberOfPages: number }> {
  const prisma = new PrismaClient();
  let filter = {};
  if (tagFilter) {
    filter = { where: { tags: { some: { name: { contains: tagFilter } } } } };
  }
  const numberOfPosts = await prisma.post.count(filter);
  const numberOfPages = Math.ceil(numberOfPosts / PAGE_SIZE);
  const posts = (
    await prisma.post.findMany({
      ...filter,
      omit: { contentMarkdown: true, contentRendered: true },
      include: { tags: { select: { name: true } } },
      orderBy: { publishedDate: "desc" },
      skip: PAGE_SIZE * (pageNumber - 1),
      take: PAGE_SIZE,
    })
  ).map((obj) => {
    return { ...obj, tags: obj.tags.map((tag) => tag.name) };
  });
  return { metadata: posts, numberOfPages: numberOfPages };
}
